<?php
/**
 * Created by PhpStorm.
 * User: Administrator
 * Date: 2019/4/19
 * Time: 23:41
 */
define('ADDONS_PATH', DOXCX_FRAME_PATH . 'Addons/');
define('IS_WIN', strstr(PHP_OS, 'WIN') ? 1 : 0);
require_once '../../vendor/autoload.php';
require_once DOXCX_FRAME_PATH . 'conf.php';
spl_autoload_register('selfLoader::autoload', true, true);
register_shutdown_function('selfLoader::fatalError');
set_error_handler('selfLoader::myErrorHandler');
set_exception_handler('selfLoader::appException');

class selfLoader
{
    public static function autoload($class)
    {

        //trait 逻辑类    //services 服务类
        $first_namespace = strstr($class, '\\', true);
        if (in_array($first_namespace, ['Traits', 'Services', 'Models'])) {
            $file = DOXCX_FRAME_PATH . str_replace('\\', DIRECTORY_SEPARATOR, $class) . '.php';
            if (is_file($file)) {
                // Win环境下面严格区分大小写
                if (IS_WIN && false === strpos(str_replace('/', '\\', realpath($file)), $class . '.php')) {
                    return false;
                }

                includeFile($file);
                return true;
            }
        }
        if (!$first_namespace === 'Addons') return false;
        $class = ltrim($class, 'Addons\\');
        $file = ADDONS_PATH . str_replace('\\', DIRECTORY_SEPARATOR, $class) . '.php';

        if (is_file($file)) {
            // Win环境下面严格区分大小写
            if (IS_WIN && false === strpos(str_replace('/', '\\', realpath($file)), $class . '.php')) {
                return false;
            }

            includeFile($file);
            return true;
        }


    }

    //设置错误日志输出
    public static function myErrorHandler($errno, $errstr, $errfile, $errline)
    {
        require_once DOXCX_LIB . 'sys/log.php';
        $err_msg = "信息：{$errno}。内容：{$errstr},发生在文件{$errfile}的{$errline}行";
        switch ($errno) {
            case E_USER_ERROR:
                Log::error($err_msg);
                if (Conf::$is_developing) {
                    echo $err_msg;
                }
                break;
            case E_USER_WARNING:
                Log::warn($err_msg);
                break;
            case E_USER_NOTICE:
                Log::info($err_msg);
                break;
            default:
                Log::info($err_msg);
                break;
        }
        return true;
    }

    /**
     * 自定义异常处理
     * @access public
     * @param mixed $e 异常对象
     */
    public static function appException($e)
    {
        require_once DOXCX_LIB . 'sys/log.php';
        $error = array();
        $error['message'] = $e->getMessage();
        $trace = $e->getTrace();
        if ('E' == $trace[0]['function']) {
            $error['file'] = $trace[0]['file'];
            $error['line'] = $trace[0]['line'];
        } else {
            $error['file'] = $e->getFile();
            $error['line'] = $e->getLine();
        }
        $error['trace'] = $e->getTraceAsString();

        \Log::error($error['message']);
        // 发送404信息
//        header('HTTP/1.1 404 Not Found');
//        header('Status:404 Not Found');

        self::halt($error);
    }

    // 致命错误捕获
    public static function fatalError()
    {
        // Log::save();
        if ($e = error_get_last()) {
            switch ($e['type']) {
                case E_ERROR:
                case E_PARSE:
                case E_CORE_ERROR:
                case E_COMPILE_ERROR:
                case E_USER_ERROR:
                    ob_end_clean();
                    self::halt($e);
                    break;
            }
        }
    }

    /**
     * 错误输出
     * @param mixed $error 错误
     * @return void
     */
    public static function halt($error)
    {
    	require_once DOXCX_LIB . 'sys/log.php';
        $e = array();
        // if (APP_DEBUG||IS_CLI) {
        if (Conf::$is_developing) {

            //调试模式下输出错误信息
            if (!is_array($error)) {
                $trace = debug_backtrace();
                $e['message'] = $error;
                $e['file'] = $trace[0]['file'];
                $e['line'] = $trace[0]['line'];
                ob_start();
                debug_print_backtrace();
                $e['trace'] = ob_get_clean();
            } else {
                $e = $error;
            }

            // if(IS_CLI){
            //     exit(iconv('UTF-8','gbk',$e['message']).PHP_EOL.'FILE: '.$e['file'].'('.$e['line'].')'.PHP_EOL.$e['trace']);
            // }
        } else {
            // 否则定向到错误页面
//            $error_page         = C('ERROR_PAGE');

//            if (!empty($error_page)) {
//                die('服务器出现错误');
//            }
//            else {

            $message = is_array($error) ? $error['message'] : $error;
            $e['message'] = '服务器出现错误';
//                $e['message']   = C('SHOW_ERROR_MSG')? $message : C('ERROR_MESSAGE');
//            }
        }
        \Log::error(is_array($e) ? implode("\n",$e) : $e);
        // 包含异常页面模板
        $exceptionFile = DOXCX_LIB . 'sys/exception.tpl';
        include $exceptionFile;
        exit;
    }

}

if (!function_exists('includeFile')) {
    function includeFile($file)
    {
        include $file;
    }
}
//larael 注册
$database = [
    'driver' => 'mysql',
    'host' => Conf::$db_host,
    'database' => Conf::$db_dbname,
    'username' => Conf::$db_username,
    'port' => Conf::$db_port ? Conf::$db_port : 3306,
    'password' => Conf::$db_password,
    'charset' => 'utf8',
    'collation' => 'utf8_unicode_ci',
    'prefix' => Conf::$db_tablePrefix,
    'strict' => true,
];

$capsule = new Illuminate\Database\Capsule\Manager;
// 创建链接
$capsule->addConnection($database);

// 设置全局静态可访问DB
$capsule->setAsGlobal();

// 启动Eloquent
$capsule->bootEloquent();
/**
 * in_array is too slow when array is large
 */
//function inArray($item, $array) {
//    $str = implode(',', $array);
//    $str = ',' . $str . ',';
//    $item = ',' . $item . ',';
//    return false !== strpos($item, $str) ? true : false;
//}
/**
 * 将laravel database 转换成全局的 db 类
 * Class LDB
 */
//class LDB extends \Illuminate\Database\Capsule\Manager{
//
//
//
//}
class_alias("\\Illuminate\\Database\\Capsule\\Manager", 'LDB');//转换成全局
// 读取扩展配置文件
// if (is_dir(DOXCX_FRAME_PATH . 'Addons')) {
//     $addons_dir = DOXCX_FRAME_PATH . 'Addons';
//     $dirs = scandir($addons_dir);
//     foreach ($dirs as $dir) {
//         $path = $addons_dir . '/' . $dir;
//         if (is_dir($path) && $dir != '.' && $dir != '..') {
//             if (is_file($path . '/function.php')) {
//                 includeFile($path . '/function.php');
//             }
//         }
//     }
// }

/**
 * 自定义返回成功JSON的方法
 * @param string $msg
 * @param null $data
 * @param array $exts
 * @return array
 */
function success($msg = 'OK', $data = null, $exts = [])
{
    $res = [
        'status' => 'success',
        'msg' => $msg,
        'data' => $data,
    ];
    foreach ($exts as $k => $ext) {
        $res[$k] = $ext;
    }
    return $res;
}

/**
 * 自定义返回失败JSON的方法
 * @param string $msg
 * @param null $data
 * @param array $exts
 * @return array
 */
function error($msg = 'OK', $data = null, $exts = [])
{
    $res = [
        'status' => 'error',
        'msg' => $msg,
        'data' => $data,
    ];
    foreach ($exts as $k => $ext) {
        $res[$k] = $ext;
    }
    return $res;
}

/**
 * 　　* 下划线转驼峰
 * 　　* 思路:
 * 　　* step1.原字符串转小写,原字符串中的分隔符用空格替换,在字符串开头加上分隔符 --修改为不转小写
 * 　　* step2.将字符串中每个单词的首字母转换为大写,再去空格,去字符串首部附加的分隔符.
 * 　　*/
function camelize($uncamelized_words, $separator = '_')
{
    $uncamelized_words = $separator . str_replace($separator, " ", $uncamelized_words);
    return ltrim(str_replace(" ", "", ucwords($uncamelized_words)), $separator);
}

/**
 * 　　* 驼峰命名转下划线命名
 * 　　* 思路:
 * 　　* 小写和大写紧挨一起的地方,加上分隔符,然后全部转小写
 * 　　*/
function uncamelize($camelCaps, $separator = '_')
{
    return preg_replace('/([a-z])([A-Z])/', "$1" . $separator . "$2", $camelCaps);
}

includeFile('BaseController.php');
