
function convertDeviceStructs(devices) {
    var realDevices = [];
    if (devices && devices.length > 0) {
        for (var i in devices) {
            var {deviceName, localName, deviceId, RSSI, advertisData} = devices[i];
            realDevices.push({'name': deviceName || localName, 'deviceId': deviceId, rssi: RSSI, advertisData });
        }
    }
    return realDevices;
}

function convertCharacteristics(characteristics) {
    var realCharacteristics = [];
    if (characteristics && characteristics.length > 0) {
        for (var i in characteristics) {
            var {characteristicId, unused_serviceId, unused_value, properties} = characteristics[i];
            realCharacteristics.push({'uuid': characteristicId, 'properties': properties });
        }
    }
    return realCharacteristics;
}

var wx = {
    chooseImage({ count, unused_sizeType, sourceType, success, fail, complete }) {
        my.chooseImage({
            'count': count, 'sourceType': sourceType, 'success': ({ apFilePaths, unused_tempFiles }) => {
                success && success({ tempFilePaths: apFilePaths });
            }, 'fail': fail, 'complete': ({ apFilePaths, unused_tempFiles }) => {
                complete && complete({ tempFilePaths : apFilePaths });
            }
        });
    },

    saveImageToPhotosAlbum({ filePath, success, fail, complete }) {
        my.saveImage({ 'url': filePath, 'showActionSheet': true, 'success': success, 'fail': fail, 'complete': complete });
        console.warn('不支持的能力1');
    },

    startRecord({ success, fail, complete }) {
        my.startRecord({
            'maxDuration': 60, 'minDuration': 1, 'success': ({ tempFilePath }) => {
                success && success({ apFilePath: tempFilePath });
            }, fail, 'complete': ({ tempFilePath }) => {
                complete && complete({ apFilePath: tempFilePath });
            }
        });
        console.warn('不支持的能力2');
    },

    getBackgroundAudioPlayerState({ success, fail, complete }) {
        my.getBackgroundAudioPlayerState({ 'success': success });
        if (fail || complete) {
            console.warn('不支持的能力3');
        }
    },

    playBackgroundAudio({ dataUrl, title, coverImgUrl, success, fail, complete }) {
        my.playBackgroundAudio({
            'dataUrl': dataUrl, 'title': title, 'singer': null,
            'describe': null, 'logo': null, 'cover': coverImgUrl, 'success': success,
            'fail': fail,
            'complete': complete
        });
        console.warn('不支持的能力4');
    },

    chooseVideo({ sourceType, maxDuration, camera, success, fail, complete }) {
        my.chooseVideo({
            'maxDuration': maxDuration, 'sourceType': sourceType, 'camera': camera, 'success': ({ tempFilePath, duration, size, height, width }) => {
                success && success(tempFilePath, duration, size, width, height);
            }, 'fail': fail, 'complete': complete
        });
    },

    createVideoContext(videoId) {
        var videoContext = {
            _delegate: my.createVideoContext(videoId),
            play: function () {
                this._delegate.play();
            },
            pause: function () {
                this._delegate.pause();
            },
            seek: function (position) {
                this._delegate.seek(position);
            },
            sendDanmu: function ({ unused_text, unused_color }) {
                console.error('不支持的能力5');
            },
            playbackRate: function (unused_rate) {
                console.error('不支持的能力5');
            },
            requestFullScreen: function () {
                console.error('不支持的能力5');
            },
            exitFullScreen: function () {
                console.error('不支持的能力5');
            }
        };
        return videoContext;
    },

    uploadFile({ url, filePath, name, header, formData, success, fail, complete }) {
        my.uploadFile({
            'url': url, 'filePath': filePath, 'fileName': name,
            'fileType': 'image/video', 'headers': header, 'formData': formData, 'success': ({ data, statusCode }) => {
                success && success(data, statusCode);
            }, 'fail': fail, 'complete': complete
        });
        console.warn('不支持的能力6');
    },

    downloadFile({ url, header, success, fail, complete }) {
        my.downloadFile({ 'url': url, 'headers': header }, ({ tempFilePath }) => {
            success && success({ apFilePath: tempFilePath });
        });
        if (fail || complete) {
            console.warn('不支持的能力7');
        }
    },

    createMapContext(mapId) {
        var mapContext = {
            _delegate: my.createMapContext(mapId),
            getCenterLocation: function (options) {
                return this._delegate.getCenterLocation(options);
            },
            moveToLocation: function () {
                this._delegate.moveToLocation();
            },
            translateMarker: function (unused_param) {
                console.error('不支持的能力8');
            },
            includePoints: function (unused_param) {
                console.error('不支持的能力9');
            },
            getRegion: function (unused_param) {
                console.error('不支持的能力10');
            },
            getScale: function (unused_param) {
                console.error('不支持的能力11');
            }
        };
        return mapContext;
    },

    getSystemInfo({ success, fail, complete }) {
        my.getSystemInfo({
            'success': ({ model, pixelRadio, windowWidth, windowHeight, language, version,
            }) => {
                success && success({
                    'model': model, 'pixelRatio': pixelRadio,
                    'screenWidth': undefined, 'screenHeight': undefined, 'windowWidth': windowWidth, 'windowHeight': windowHeight, 'language': language, 'version': version
                });
            }, 'fail': fail, 'complete': complete
        });
        console.warn('不支持的能力12');
    },

    getNetworkType({ success, fail, complete }) {
        my.getNetworkType({
            'success': ({ unused_networkAvailable, networkType }) => {
                success && success({ 'networkType': networkType });
            }, 'fail': fail, 'complete': complete
        });
    },

    onAccelerometerChange(callback) {
        my.onAccelerometerChange({'success': callback});
    },

    onCompassChange(callback) {
        my.onCompassChange({'success': callback});
    },

    scanCode({onlyFromCamera, success, fail, complete}) {
        if (onlyFromCamera) {
             console.warn('不支持的能力13');
        }
        my.scan({'type': 'qr', 'success': ({result, unused_canType, unused_charSet, unused_path }) => {
            success && success(result);
        }, 'fail': fail, 'complete': complete});
        console.warn('不支持的能力14');
    },



    onBluetoothAdapterStateChange(callback) {
        my.onBluetoothAdapterStateChange({'success': ({available, connected}) => {
           callback && callback({'available': available, 'discovering': !connected});
        }});
    },

    startBluetoothDevicesDiscovery({services, allowDuplicatesKey, interval, success, fail, complete }) {
       
        my.startBluetoothDevicesDiscovery({'services': services, 'success': ({services}) => {
            success && success({'errMsg': services});
        }, fail, complete}); 
        console.warn('不支持的能力15');
    },

    getBluetoothDevices({ success, fail, complete }) {
        my.getBluetoothDevices({'services':[], 'success': ({devices}) => {
                var realDevices = convertDeviceStructs(devices);
                success && success({'devices': realDevices});
            }, 
            'fail': fail, 'complete': complete });
        console.warn('不支持的能力16');
    },

    getConnectedBluetoothDevices({ services, success, fail, complete }) {
        my.getConnectedBluetoothDevices({'services': services, 'success': ({ devices, errMsg }) => {
            var realDevices = convertDeviceStructs(devices);
            success && success({'devices': realDevices});
        }, 'fail': fail, 'complete': complete});   
        console.warn('不支持的能力17');
    },

    onBluetoothDeviceFound(callback) {
        my.onBluetoothDeviceFound({'success': callback});
    },

    getBLEDeviceCharacteristics({ deviceId, serviceId, success, fail, complete }) {
        my.getBLEDeviceCharacteristics(deviceId, serviceId, ({ characteristics }) => {
            success(convertCharacteristics(characteristics));
        }, fail, complete);  
    },

    readBLECharacteristicValue({ deviceId, serviceId, characteristicsId, success, fail, complete }) {
        my.readBLECharacteristicValue({'deviceId': deviceId, 'serviceId': serviceId, 
            'characteristicsId': characteristicsId, 'descriptorId': '00002902-0000-1000-8000-00805f9b34fb',
            'success': ({ characteristic }) => {
                success && success({'characteristic': characteristic }); 
            }, fail, complete}); 
        console.warn('不支持的能力18');
    },

    writeBLECharacteristicValue({ deviceId, serviceId, characteristicId, value, success, fail, complete }) {
        my.writeBLECharacteristicValue({'deviceId': deviceId, 'serviceId': serviceId, 'characteristicId': characteristicId, 'descriptorId': '00002902-0000-1000-8000-00805f9b34fb',
            'value': value, 'success': success, 'fail': fail, 'complete': complete});
        console.warn('不支持的能力19');
    },

    notifyBLECharacteristicValueChange({ deviceId, serviceId, characteristicId, state, success, fail, complete }) {
        my.notifyBLECharacteristicValueChange({'deviceId': deviceId, 'serviceId': serviceId, 'characteristicId':characteristicId, 'descriptorId': '00002902-0000-1000-8000-00805f9b34fb',
            'success': success, 'fail': fail, 'complete': complete});
        console.warn('不支持的能力20');
    },

    getStorageSync(key) {
        var value = my.getStorageSync({'key': key});
        return value ? value.data : null;
    },

    onBLEConnectionStateChange(callback) {
        my.onBLEConnectionStateChanged({'success': callback});
    },

    vibrateLong(object) {
        my.vibrate(object);
        console.warn('不支持的能力21');
    },

    setStorageSync(key, value) {
        my.setStorageSync({
            'key': key,
            'data': value
        });
    },

    vibrateShort(object) {
        my.vibrate(object);
        console.warn('不支持的能力22');
    },

    showToast({ title, icon, image, duration, mask, success, fail, complete }) {
        var type = icon === 'success' ? 'success' : 'none';
        my.showToast({'content': title, 'type': type, 'duration': duration, 'success': success, 'fail': fail, 'complete': complete});
        console.warn('不支持的能力23');
    },

    showLoading({ title, mask, success, fail, complete }) {
        my.showLoading({'content': title, 'delay': 0, 'success': success, 'fail': fail, 'complete': complete});
        console.warn('不支持的能力24');
    },

    showActionSheet({ itemList, itemColor, success, fail, complete }) {
        my.showActionSheet({'title':'', 'items': itemList, 'cancelButtonText':'取消', destructiveBtnIndex: 0, success: ({ index }) => {
            success({tapIndex: index});
        }, fail: fail, complete: complete});
        console.warn('不支持的能力25');
    },

    setNavigationBarTitle({ title, success, fail, complete }) {
        my.setNavigationBar({title:title, image: null, backgroundColor: null, borderBottomColor: null,     reset: false, success: success, fail: fail, complete: complete});
        console.warn('不支持的能力26');
    },

    canvasToTempFilePath({ x, y, width, height, destWidth, destHeight, canvasId, success, fail, complete }) {
        const ctx = my.createCanvasContext(canvasId)
        ctx.toTempFilePath({
            success: success
        });
    },

    request({ url, data, header, method, dataType, success, fail, complete }) {
        my.httpRequest({url: url, headers: header, method: method, data: JSON.stringify(data), timeout: 999000, dataType: dataType, success: ({ data, status, headers }) => {
            success && success({data:data, statusCode: status, header: headers});
        }, fail: fail, complete: complete});
        console.warn('不支持的能力27');
    },

    connectSocket({ url, data, header, method, protocols, success, fail, complete }) {
        my.connectSocket({url: url, data:data, header: header, method:method, success: success});
    },

    sendSocketMessage({ data, success, fail, complete }) {
        my.sendSocketMessage({data: data, success: success});
    }
};
module.exports = wx;